##############################################################################
# $Id: bootservers.pl,v 1.2 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Subroutines for dealing with Jumpstart boot servers.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created ??/??/97
# last modified 04/09/98
##############################################################################
# $Log: bootservers.pl,v $
# Revision 1.2  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.1  1999/08/09 02:52:58  jheiss
# Initial revision
#
##############################################################################

require 'basedir.pl';
require 'can.pl';
require 'facts.pl';
require 'inarray.pl';
require 'rsh.pl';
require 'subnet.pl';

#############
# Constants #
#############

$SILENT = -1;
$NORMAL = 0;
$VERBOSE = 1;
$FORCE = 1;
$NOFORCE = 0;

##############################################################
# Subroutine that returns a list of all of the boot servers. #
##############################################################
sub bootservers
{
	#############
	# Constants #
	#############
	$CACHEFILE = "$LOGBASEDIR/bootcache";
	$OLDCACHEFILE = "$LOGBASEDIR/bootcache-";
	$NEWCACHEFILE = "$LOGBASEDIR/bootcache.new";

	###################
	# Other variables #
	###################
	my @bootservers;
	my ($silent, $verbose);
	my $starttime = time();

	#################################################
	# Get any options the user might have passed us #
	#################################################
	($output_opt, $force_opt) = @_;

	if ($output_opt == $SILENT)
	{
		$silent = $TRUE;
		$verbose = $FALSE;
	}
	elsif ($output_opt == $VERBOSE)
	{
		$verbose = $TRUE;
		$silent = $FALSE;
	}
	else
	{
		$verbose = $FALSE;
		$silent = $FALSE;
	}

	########################################################
	# List out any boot servers which we know we can't use #
	########################################################
	my @BADBOOTS = ('hsca025');  # Model office sandbox app server

	####################################################################
	# List out any boot servers which don't match the regular          #
	# application or network server naming conventions or would        #
	# otherwise fail our checks below.  These are not checked as       #
	# thouroghly so make sure they are good.  A server listed here     #
	# take precedence over other possible servers on that subnet.      #
	#                                                                  #
	# Note that if you make a change here and want it to be effective  #
	# immediately you should remove the cached list ($CACHEFILE) of    #
	# boot servers.                                                    #
	####################################################################

	# hsca019 is the funky model office appserver.  We want to use it
	# anyway so we put it here.
	my @OTHERBOOTS = ('hsca019');

	##################################################################
	# Check to see if our cached copy is less than a day old and use #
	# it if that is the case.  (Presuming the user didn't force us   #
	# to rebuild the cache.)                                         #
	##################################################################
	if (-r $CACHEFILE && ! $force_opt)
	{
		open(CACHE, $CACHEFILE);
		chop(my $cachetime = <CACHE>);
		if (defined($cachetime) && $cachetime >= (time() - $SEC_IN_DAY))
		{
			print "Using cached list\n" unless ($silent);
			while (<CACHE>)
			{
				chop;
				push(@bootservers, $_);
			}

			close(CACHE);
			return(@bootservers);
		}
		close(CACHE);
	}

	####################################################################
	# If we get here it means that either the cache file doesn't exist #
	# or is too old.  So we create one.                                #
	####################################################################
	print "No cached list, we will create one (Sorry, this is slow)\n"
		unless ($silent);

	# Open new cache for writing.
	open(NEWCACHE, ">$NEWCACHEFILE");

	# Read in all possible boot servers
	my @possible_boots;
	open(HOSTS, "ypcat hosts.byaddr|");
	while(<HOSTS>)
	{
		my ($ip, $host) = split;

		if ($host =~ /(^hsca\d)|(^hscns\d)/ &&
			(! ($host =~ /-/)) &&  # Skip things like hscns01-qe1
			(! inarray($host, @BADBOOTS)))  # Skip bad boot servers
		{
			push(@possible_boots, $host);
		}
	}
	close(HOSTS);

	# Now sort the list of possible boot servers.  We want to put
	# the network servers before the app servers because over time
	# we want to stop using the app servers as boot servers.
	@possible_boots = reverse sort @possible_boots;

	my @subnets_picked;
	foreach $host (@possible_boots)
	{
		print "Checking $host\n" if ($verbose);

		# First make sure that we haven't already found an
		# acceptable app server on that subnet.
		if (inarray(subnet($host), @subnets_picked))
		{
			print "  Already have one on that subnet\n" if ($verbose);
		}
		else
		{
			print "  Don't have one on that subnet\n" if ($verbose);

			my $goodboot = 0;

			if (can_ping($host) && can_rsh($host))
			{
				print "    Can ping and rsh to it\n" if ($verbose);

				# Check to make sure it is a 1.6 or greater app server or
				# a 1.0 or greater network server.
				if ($host =~ /^hsca\d/ &&
					rsh_noerr($host,
							"ls /usr/local/appserver_version_*") =~
                                 /appserver_version_(\d\.\d+)/ && $1 >= 1.6)
				{
					print "    App server version $1 is good\n" if ($verbose);
					$goodboot = 1;
				}
				elsif ($host =~ /^hscns\d/ &&
						rsh_noerr($host,
								"ls /usr/local/netsvr_version_*") =~
									/netsvr_version_(\d\.\d+)/ && $1 >= 1.0)
				{
					print "    Network server version $1 is good\n"
						if ($verbose);
					$goodboot = 1;
				}
			}

			if ($goodboot)
			{
				print "    Keeping $host\n" if ($verbose);
				push(@bootservers, $host);
				foreach $interface (all_interfaces($host))
				{
					push(@subnets_picked, subnet($interface));
				}
			}
		}
	}

	# Push on the non-standard ones too
	push(@bootservers, @OTHERBOOTS);

	print "Saving " . scalar(@bootservers) . " bootservers to cache\n"
		if ($verbose);

	# Save the current time as the first line of the cache file
	print NEWCACHE time() . "\n";

	# Now put all of the boot servers in there
	foreach $bs (@bootservers)
	{
		print NEWCACHE "$bs\n";
	}
	close(NEWCACHE);

	# Copy new cache file into place
	system("cp $CACHEFILE $OLDCACHEFILE") if (-e $CACHEFILE);
	system("cp $NEWCACHEFILE $CACHEFILE");

	print "Elapsed time is " . (time() - $starttime) . " seconds\n"
		if ($verbose);

	# And return the list
	return sort(@bootservers);
}

return 1;

