##############################################################################
# $Id: checkrules.pl,v 1.3 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Subroutine which checks a machine's hardware against a Jumpstart rules.ok
# file to determine if the machine matches any of the rules.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 08/??/97
# last modified 04/27/98
##############################################################################
# $Log: checkrules.pl,v $
# Revision 1.3  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.2  1999/08/09 03:58:03  jheiss
# Fixed typo
#
# Revision 1.1  1999/08/09 03:02:35  jheiss
# Initial revision
#
##############################################################################

require 'info.pl';
require 'rsh.pl';

# 0:  No debugging
# 1:  Info about information collected from machine
# 2:  Prints each rule (and part thereof) as it is checked and whether
#     it succeeded or failed
my $DEBUGLEVEL = 0;

sub checkrules
{
	my $host = $_[0];
	#my $arch = $_[1];
	#my $osver = $_[2];
	my $profile_directory = $_[3];

	die "Wrong args to \&checkrules" unless (defined($host) &&
		defined($profile_directory));

	my $os = get_os($host);
	my $osver = get_os_version($host);
	my $osmajor = get_os_major_version($host);

	# This subroutine gathers a bunch of information from the target
	# machine in very OS specific fashion.  As such we have to be a
	# bit picky about what versions of SunOS/Solaris are running.
	unless ($os eq 'SunOS' && ($osmajor == 4 || $osmajor == 5))
	{
		print "\&checkrules requires SunOS 4.x or 5.x, $host is $os $osver\n";
		return 0;
	}

	# Initialize some variables
	my @rules = ();

	debug_print("\n", 1);

	###################
	# Load rules file #
	###################

	if (! -e "$profile_directory/rules.ok")
	{
		die "Missing rules file:  $profile_directory/rules.ok";
	}

	my $rulestemp = '';
	local *RULES;
	open(RULES, "$profile_directory/rules.ok");
	RULELINE: while(<RULES>)
	{
		# Read in the rules file to an array, concatenating
		# any rules continued over more than one line.

		next RULELINE if /^#/;  # Skip comments
		chop;  # Hack off newline
		next RULELINE if /^\s*$/;  # Skip blank lines

		# Check for a continuation (\) character at the end of the line
		if (/\\\s*$/)
		{
			s/\s*$//;  # Remove any trailing whitespace
			chop;  # Remove backslash
			$rulestemp .= $_;
		}
		elsif ($rulestemp)
		{
			$rulestemp .= $_;
			push(@rules, $rulestemp);
			$rulestemp = '';
		}
		else
		{
			push(@rules, $_);
		}
	}
	close(RULES);

	###########################################
	# Gather hard drive information from host #
	###########################################

	# This bunch of code should be moved to get_hard_disks in info.pl.

	# A number of the rules check whether certain disks exist and are the
	# right size.  To avoid having to go and get this information for each
	# rule we collect that info ahead of time.
	#
	# A lot of the info about the disks in the machine is gathered from
	# the list of disks that format spits out when you first start it
	# up.  Since we run through that info several times, we go ahead and
	# grab it once so that we don't have to make multiple connections to
	# the machine for the same info.
	#
	# Format is designed to be interactive.  The echo makes it
	# bail out like we want.  Note that format occasionally takes
	# an exceptionally long time to run.  I suspect that this
	# occurs due to hardware problems on the machine.  That is another
	# benefit for only doing this once.

	my @disks = ();
	my %disksize;

	my @formatlines = ();
	local *FORMAT;
	if ($osmajor == 5)
	{
		open(FORMAT, "echo | rsh $host /usr/sbin/format |");
	}
	elsif ($osmajor == 4)
	{                                           
		open(FORMAT, "echo | rsh $host /usr/etc/format |");
	}
	else
	{
		# How'd we get here?
		die "Internal error in \&checkrules";
	}

	while(<FORMAT>)
	{
		chop;
		push(@formatlines, $_);
	}
	close(FORMAT);

	# First we get all of the available disks
	foreach $_ (@formatlines)
	{
		if (/^\s+\d+\./)
		{
			my ($junk, $disk) = split;
			push(@disks, $disk);
			debug_print("Found disk: $disk\n", 1);
		}
	}

	if ($osmajor == 5)
	{
		my $disk;
		foreach $disk (@disks)
		{
			# Once again stolen from suninstall.  We know which
			# disk we are looking for, but prtvtoc wants a slice as
			# well.  Thus we iterate through slices 0-7 until we
			# find one that works.  It doesn't matter which slice
			# we use, we just need one that exists.

			# A possible speed-up would be to just assume slice 2
			# exists and only search if it doesn't...
	
			my ($i, $slice);
			SLICE: for ($i = 0 ; $i < 8 ; $i++)
			{
				# Also, Solaris 2.3 requires that you use the
				# raw device (/dev/rdsk) while 2.4 or greater
				# allow either the raw device or the block
				# device (/dev/dsk).  As such we use the raw
				# device.
				unless (rsh_return($host, "prtvtoc /dev/rdsk/${disk}s$i"))
				{
					$slice = $i;
					last SLICE;
				}
			}
	
			my ($bps, $spc, $cyl);
			open(PRTVTOC, "rsh $host prtvtoc /dev/rdsk/${disk}s${slice}|");
			while(<PRTVTOC>)
			{
				if (/bytes\/sector/)
				{
					(my $junk, $bps) = split;
				}
				elsif (/sectors\/cylinder/)
				{
					(my $junk, $spc) = split;
				}
				elsif (/cylinders/)
				{
					(my $junk, $cyl) = split;
				}
			}
			close(PRTVTOC);
			
			my $mbytes = ($bps * $spc * $cyl) / 1048576;
	
			$disksize{$disk} = $mbytes;
		}
	}
	elsif ($osmajor == 4)
	{
		my $disk;
		foreach $disk (@disks)
		{
			foreach $_ (@formatlines)
			{
				next unless (/^\s+${disk}:/);

				# A typical line like what we are looking for:
				# sd0: <SUN1.05 cyl 2036 alt 2 hd 14 sec 72>
				# And I saw one like this once (with the ^[
				# being the actual escape character):
				# sd0: <FUJITSU 2624FA\^[ cyl 1463 alt 2 ...
				# As such, I no longer trust just doing a split
				# and assuming things will be where I expect.

				/cyl ([0-9]+) alt [0-9]+ hd ([0-9]+) sec ([0-9]+)/;
				$cyl = $1;
				$heads = $2;
				$sec = $3;

				# We assume 512 bytes/sector
				$disksize{$disk} = (512 * $sec * $heads * $cyl) / 1048576;
			}
		}
	}
	else
	{
		# Acck, what happened?
		die "Internal error in \&checkrules"
	}

	# For SunOS machines we have to do a little name translation.
	# Namely, we need to map the drive names from SunOS format
	# (sd0, sd1) to Solaris format (c0t3d0, c0t1d0).  But the trick
	# is that we need to know which SCSI ids the various SunOS names
	# map to in order to correctly map them to the Solaris names.
	# The most reliable way to get this is from format.  Here is a
	# snippit of the output from format on a machine with a pretty
	# wacky device name to SCSI id mapping:
	#
	#        0. sd0 at esp0 slave 24
	#           sd0: <FUJITSU 2624FA\cyl 1463 alt 2 hd 11 sec 63>
	#        1. sd1 at esp0 slave 0
	#           sd1: <SUN1.3G cyl 1965 alt 2 hd 17 sec 80>
	#        2. sd2 at esp0 slave 32
	#           sd2: <SUN1.3G cyl 1965 alt 2 hd 17 sec 80>
	#        3. sd4 at esp0 slave 8
	#           sd4: <SUN0424 cyl 1151 alt 2 hd 9 sec 80>
	#        4. sd5 at esp0 slave 16
	#           sd5: <SUN0669 cyl 1614 alt 2 hd 15 sec 54>
	#        5. sd6 at esp0 slave 48
	#           sd6: <SUN4.2G cyl 3880 alt 2 hd 16 sec 135>
	#
	# It turns out that if you divide the slave number (i.e. 24 for
	# sd0) by 8 that is the SCSI id number.  Intuitive, eh?  :)

	if ($osmajor == 4)
	{
		my %SUNTOSOL;

		foreach $_ (@formatlines)
		{
			next unless (/\s+\d+\./);
			/(sd\d+).*slave\s+(\d+)$/;
			my $disk = $1;
			my $slave = $2;
			my $scsi_id = $slave / 8;
			debug_print("Mapping $disk to c0t${scsi_id}d0\n", 1);
			$SUNTOSOL{$disk} = "c0t${scsi_id}d0";
		}

		my ($disk, @newdisks);
		foreach $disk (@disks)
		{
			push(@newdisks, $SUNTOSOL{$disk});
		}
		@disks = @newdisks;

		foreach $disk (keys(%disksize))
		{
			$disksize{$SUNTOSOL{$disk}} = $disksize{$disk};
		}
	}

	my $disk;
	foreach $disk (@disks)
	{
		debug_print("$disk: $disksize{$disk}\n", 1);
	}
	
	#####################################
	# Check hardware against rules file #
	#####################################

	# See the example rules file that Sun provides for more complete
	# information about the rules file syntax.  We actually only support
	# a subset of it here.
	#
	# Basically each rule in the rules file consists of 1 or more requirements
	# followed by the begin script, profile and finish script entries.
	# Multiple requirements are seperated by '&&'.  Nothing seperates the
	# last requirement from the begin, profile and finish entries.  Those
	# entries are required however so we can just assume that the last three
	# words on the line are them.  Each requirement consists of a keyword
	# followed by one or more values.
	#
	# The requirement keywords that we support are:
	# - any
	# - domainname
	# - disksize
	# - hostname
	# - karch
	# - memsize
	# - model
	# - network
	# 
	# The requirement keywords that we don't support are:
	# - arch
	# - installed
	# - totaldisk
	#
	# We also do not currently support the negation operator (!).
	
	my $rule;
	RULE: foreach $rule (@rules)
	{
		# Remove the begin script, profile and finish script entries
		my (@ruleparts) = split(' ', $rule);
		splice(@ruleparts, -3);
		$rule = join(' ', @ruleparts);

		debug_print("\nRule: $rule\n", 2);

		# Now split the rule up into the requirements
		my (@requirements) = split(/&&/, $rule);

		# Now iterate through each requirement checking to see if it
		# passes.
		REQ: foreach $requirement (@requirements)
		{
			my ($keyword, @values) = split(' ', $requirement);

			debug_print("  Keyword: $keyword\n", 2);

			if ($keyword eq 'any')
			{
				# any always passes
				next REQ;
			}
			elsif ($keyword eq 'domainname')
			{
				my ($ruledomain) = @values;

				if ($ruledomain eq get_nis_domainname($host))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'disksize')
			{
				my ($device, $range) = @values;

				if (inarray($device, @disks) &&
					inrange($disksize{$device}, $range))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'hostname')
			{
				my ($rulehost) = @values;

				if ($rulehost eq $host)
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'karch')
			{
				my ($rulekarch) = @values;

				if ($rulekarch eq get_architecture($host))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'memsize')
			{
				my ($memrange) = @values;

				if (inrange(get_ram($host), $memrange))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'model')
			{
				my ($rulemodel) = @values;

				if ($rulemodel eq get_model($host))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
			elsif ($keyword eq 'network')
			{
				my ($ruleip) = @values;

				if ($ruleip eq get_ip($host))
				{
					next REQ;
				}
				else
				{
					next RULE;
				}
			}
		}

		# If we made it here that means that all of the requirements of
		# this rule were met and thus the machine passes.  Return a non-zero
		# number to indicate this.
		debug_print("  This rule passed\n", 2);
		return 1;
	}

	# If we made it here that means that no rules were successfully
	# matched.  Return a zero value to indicate this.
	debug_print("  No rules passed\n", 2);
	return 0;

}  # End of rules checking

# This subroutine takes a debugging message and the level of debugging that
# that messages should be printed at.  If we are at that level of debugging
# or greater, we print it.
sub debug_print
{
	my $message = $_[0];
	my $level = $_[1];

	if (! defined($message))
	{
		return;
	}
	
	if (! defined($level))
	{
		$level = 1;
	}

	if ($DEBUGLEVEL >= $level)
	{
		print $message;
	}
}

# This subroutine takes a value and a range and returns whether the value
# is within the given range.  The range can also be a single value.  In
# that case we return whether the given value matches that range value.
# A non-zero return value indicates that the value is in the given range,
# a zero return value indicates that it is not or that an error occurred.
sub inrange
{
	my ($value, $range) = @_;

	unless (defined($value) && defined($range))
	{
		return 0;
	}

	if ($range =~ /-/)  # If they gave us a range
	{
		my ($lowerbound, $upperbound) = split(/-/, $range);

		if ($value < $lowerbound || $value > $upperbound)
		{
			return 0;
		}
		else
		{
			return 1;
		}
	}
	else
	{
		if ($value == $range)
		{
			return 1;
		}
		else
		{
			return 0;
		}
	}
}

return 1;

