##############################################################################
# $Id: info.pl,v 1.5 1999/08/26 04:06:57 jheiss Exp $
##############################################################################
# Various, mostly trivial, functions for getting bits of information from a
# remote machine.  The nice thing about these functions is that they cache
# their answers.  So you can just call get_os as often as you want and it will
# cache the answer for you.  Also worth mentioning is that none of these
# functions use dmesg.  In a lot of cases it is easier to get the info we want
# from dmesg than other ways but dmesg is unreliable.  dmesg has a fixed
# buffer and a number of errors (NFS server not responding is common) overflow
# that buffer.  These functions return -1 when they can't find the requested
# information.
##############################################################################
# Copyright (C) 1999  Jason Heiss (jheiss@ofb.net)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
##############################################################################
# Pre-RCS version control:
# created 03/16/98
# last modified 06/09/98
##############################################################################
# $Log: info.pl,v $
# Revision 1.5  1999/08/26 04:06:57  jheiss
# Added copyright and GNU license message.
#
# Revision 1.4  1999/08/09 04:00:02  jheiss
# Fixed a couple of typos
#
# Revision 1.3  1999/04/23 18:04:30  jheiss
# Expanded AIX memory gathering comments.
#
# Revision 1.2  1999/04/22 17:06:43  jheiss
# Added some comments for getting amount of RAM on other OSs
#
# Revision 1.1  1999/04/22 16:47:11  jheiss
# Initial revision
#
##############################################################################

# Libraries
require 'can.pl';
require 'facts.pl';
require 'inarray.pl';
require 'process.pl';
require 'rsh.pl';

# Cache variables
my %cache_can_ping;
my %cache_can_rsh;
my %cache_mac_address;
my %cache_ip_address;
my %cache_nodename;
my %cache_os;
my %cache_os_version;
my %cache_os_numeric_version;
my %cache_os_major_version;
my %cache_architecture;
my %cache_nis_server;
my %cache_is_nis_server;
my %cache_nis_domainname;
my %cache_ram;
my %cache_hard_disks;
my %cache_cpu_manufacturer;
my %cache_cpu_model;
my %cache_cpu_clock_speed;
my %cache_model;
my %cache_has_dnj_file;
my %cache_image;
my %cache_image_version;
my %cache_sendmail_version;
my %cache_sendmail_mailhub;
my %cache_sendmail_smarthost;
my %cache_hostid;

# Can we ping the machine?
sub get_can_ping
{
	my $host = $_[0];

	if (defined($cache_can_ping{$host}))
	{
		return $cache_can_ping{$host};
	}
	else
	{
		my $ping = can_ping($host);
		$cache_can_ping{$host} = $ping;
		return $ping;
	}
}

# Can we rsh to the machine and execute a command?
sub get_can_rsh
{
	my $host = $_[0];

	if (defined($cache_can_rsh{$host}))
	{
		return $cache_can_rsh{$host};
	}
	else
	{
		my $rsh = can_rsh($host);
		$cache_can_rsh{$host} = $rsh;
		return $rsh;
	}
}

# MAC (aka ethernet) address of primary ethernet interface
sub get_mac_address
{
	my $host = $_[0];

	if (defined($cache_mac_address{$host}))
	{
		return $cache_mac_address{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_mac_address{$host} = -1;
			return -1;
		}

		my $ether;
		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);
		my $arch = get_architecture($host);

		my $ifconfigout;
		my $lanscanout;
		if ($os eq 'SunOS' && $arch eq 'sun4m')
		{
			$ifconfigout = rsh($host, '/sbin/ifconfig le0');
		}
		elsif ($os eq 'SunOS' && $arch eq 'sun4u')
		{
			$ifconfigout = rsh($host, '/sbin/ifconfig hme0');
		}
		elsif ($os eq 'HP-UX' && $osmajor == 9)
		{
			$lanscanout = rsh($host, '/etc/lanscan | grep lan0');
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			$lanscanout = rsh($host, '/usr/sbin/lanscan | grep lan0');
		}
		elsif ($os eq 'Linux')
		{
			my $ifconfigout = rsh($host, '/sbin/ifconfig eth0');
			if ($ifconfigout =~ /HWaddr ([0-9a-f:]+)/)
			{
				$ether = $1;
			}
			else
			{
				$ether = -1;
			}

			# Get the IP address while we're here
			# get_ip depends on this happening so if you change this
			# remember to change get_ip
			my $ip;
			if ($ifconfigout =~ /inet addr:([0-9.]+)/)
			{
				$ip = $1;
			}
			else
			{
				$ip = -1;
			}
			$cache_ip_address{$host} = $ip;
		}
		else
		{
			$ether = -1;
		}

		if ($os eq 'SunOS' && defined($ifconfigout))
		{
			if ($ifconfigout =~ /ether ([0-9a-f:]+)/)
			{
				$ether = $1;
			}
			else
			{
				$ether = -1;
			}

			# Get the IP address while we're here
			# get_ip depends on this happening so if you change this
			# remember to change get_ip
			my $ip;
			if ($ifconfigout =~ /inet ([0-9.]+)/)
			{
				$ip = $1;
			}
			else
			{
				$ip = -1;
			}
			$cache_ip_address{$host} = $ip;
		}
		elsif ($os eq 'HP-UX' && defined($lanscanout))
		{
			# The lanscan output has the ethernet address as a hex
			# number.  We want to put the colons in to make it look
			# more conventional.  The output from lanscan looks like:
			# 
			# Hardware Station        Dev Hardware Net-Interface   NM
			# Path     Address        lu  State    NameUnit State  ID
			# 2.0.2    0x080009A7D674 0   UP       lan0     UP     4
			#
			# I've removed the last couple of columns but you get the idea.

			my ($hpath, $hexmac) = split(' ', $lanscanout);
			$hexmac =~ s/^0x//;
			my (@macparts) = unpack("a2a2a2a2a2a2", $hexmac);
			$ether = join(':', @macparts);

			# Using the above example line $ether would now
			# be 08:00:09:A7:D6:74
		}

		unless (defined($ether))
		{
			$ether = -1;
		}

		$cache_mac_address{$host} = $ether;
		return $ether;
	}
}

# IP address of the primary network interface
sub get_ip_address
{
	my $host = $_[0];

	my $os = get_os($host);
	my $osmajor = get_os_major_version($host);

	if (defined($cache_ip_address{$host}))
	{
		return $cache_ip_address{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_ip_address{$host} = -1;
			return -1;
		}

		my $ip;
		my $ifconfigout;

		if ($os eq 'SunOS' || $os eq 'Linux')
		{
			# get_mac_address caches the IP address as a side-effect
			# for SunOS and Linux machines.  It does that because both the
			# MAC and IP addresses are in the output from ifconfig.  So
			# we kill two birds with one stone that way.
			get_mac_address($host);
			$cache_ip_address{$host} = -1
				unless (defined($cache_ip_address{$host}));
			return $cache_ip_address{$host};
		}
		elsif ($os eq 'HP-UX' && $osmajor == 9)
		{
			$ifconfigout = rsh($host, '/etc/ifconfig lan0');
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			$ifconfigout = rsh($host, '/usr/sbin/ifconfig lan0');
		}
		else
		{
			$ip = -1;
		}

		if ($os eq 'HP-UX' && defined($ifconfigout))
		{
			if ($ifconfigout =~ /inet ([0-9.]+)/)
			{
				$ip = $1;
			}
			else
			{
				$ip = -1;
			}
		}

		unless (defined($ip))
		{
			$ip = -1;
		}

		$cache_ip_address{$host} = $ip;
		return $ip;
	}
}

# Whatever the host thinks is it's nodename (not the hostname of whatever
# interface we come in on).  I use uname -n because hostname isn't always
# in root's path.
sub get_nodename
{
	my $host = $_[0];

	if (defined($cache_nodename{$host}))
	{
		return $cache_nodename{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_nodename{$host} = -1;
			return -1;
		}

		chop(my $nodename = rsh_noerr($host, 'uname -n'));
		$cache_nodename{$host} = $nodename;
		return $nodename;
	}
}

# uname -s
sub get_os
{
	my $host = $_[0];

	if (defined($cache_os{$host}))
	{
		return $cache_os{$host};
	}
	else
	{
		my $os;

		if ($host eq 'localhost')
		{
			chop($os = `uname -s`);
		}
		else
		{
			unless (get_can_ping($host) && get_can_rsh($host))
			{
				$cache_os{$host} = -1;
				return -1;
			}

			chop($os = rsh_noerr($host, 'uname -s'));
		}

		$cache_os{$host} = $os;
		return $os;
	}
}

# uname -r
sub get_os_version
{
	my $host = $_[0];

	if (defined($cache_os_version{$host}))
	{
		return $cache_os_version{$host};
	}
	else
	{
		my $osver;

		if ($host eq 'localhost')
		{
			chop($osver = `uname -r`);
		}
		else
		{
			unless (get_can_ping($host) && get_can_rsh($host))
			{
				$cache_os_version{$host} = -1;
				return -1;
			}

			chop($osver = rsh_noerr($host, 'uname -r'));
		}

		$cache_os_version{$host} = $osver;
		return $osver;
	}
}

# Return a floating point number somehow related to the OS version.
# For SunOS this would be something like 5.51 for 5.5.1.  For HP-UX
# this would be something like 10.20 for B.10.20.  I think the letter
# has something to do with the machine architecture so I'm just tossing
# it out.
sub get_os_numeric_version
{
	my $host = $_[0];

	if (defined($cache_os_numeric_version{$host}))
	{
		return $cache_os_numeric_version{$host};
	}
	else
	{
		my $osver = get_os_version($host);

		# Hack out anything that isn't a number or a dot
		$osver =~ tr/0-9.//cd;

		# If there is now a leading dot, remove it
		if ($osver =~ /^\./)
		{
			(my $junk, $osver) = split(/\./, $osver, 2);
		}

		# Now remove any leading zeros
		$osver =~ s/^0+//;

		my ($start, @rest) = split(/\.+/, $osver);

		my $rest = join('', @rest);
		my $osnver = "$start.$rest";

		$cache_os_numeric_version{$host} = $osnver;
		return $osnver;
	}
}

# Return the major number of the OS version.  I.e. 5 for SunOS 5.x,
# 10 for HP-UX 10.x.
sub get_os_major_version
{
	my $host = $_[0];

	if (defined($cache_os_major_version{$host}))
	{
		return $cache_os_major_version{$host};
	}
	else
	{
		my $osnver = get_os_numeric_version($host);

		my ($major) = split(/\./, $osnver);
		$cache_os_major_version{$host} = $major;
		return $major;
	}
}

# uname -m
sub get_architecture
{
	my $host = $_[0];

	if (defined($cache_architecture{$host}))
	{
		return $cache_architecture{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_architecture{$host} = -1;
			return -1;
		}

		chop(my $arch = rsh_noerr($host, 'uname -m'));
		$cache_architecture{$host} = $arch;
		return $arch;
	}
}

sub get_nis_server
{
	my $host = $_[0];

	if (defined($cache_nis_server{$host}))
	{
		return $cache_nis_server{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_nis_server{$host} = -1;
			return -1;
		}

		chop(my $nisserv = rsh_noerr($host, 'ypwhich'));
		$cache_nis_server{$host} = $nisserv;
		return $nisserv;
	}
}

sub get_nis_domainname
{
	my $host = $_[0];

	if (defined($cache_nis_domainname{$host}))
	{
		return $cache_nis_domainname{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_nis_domainname{$host} = -1;
			return -1;
		}

		chop(my $domain = rsh_noerr($host, 'domainname'));
		$cache_nis_domainname{$host} = $domain;
		return $domain;
	}
}

sub get_is_nis_server
{
	my $host = $_[0];

	if (defined($cache_is_nis_server{$host}))
	{
		return $cache_is_nis_server{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_is_nis_server{$host} = -1;
			return -1;
		}

		if (process_check($host, 'ypserv'))
		{
			$cache_is_nis_server{$host} = $TRUE;
			return $TRUE;
		}
		else
		{
			$cache_is_nis_server{$host} = $FALSE;
			return $FALSE;
		}
	}
}

# Returns the number of megabytes of physical memory in the machine
#
# OSs which need to be tested and implemented:
# For AIX:
# - lsattr -l sys0 -E | grep realmem | awk '{printf("mem = %dK\n",$2)}'
# - lsattr -D1 sys0 | grep realmem
# - lsattr -El sys0 | grep realmem
# - bootinfo -r
#  Apparently lsdev and lscfg are not reliable
# - lscfg
# - lsdev -Cc memory
# For Linux:
# - /proc/meminfo
# - free
# For Tandem NonStop-UX:
# - prtconf  (same as Solaris)
sub get_ram
{
	my $host = $_[0];

	if (defined($cache_ram{$host}))
	{
		return $cache_ram{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_ram{$host} = -1;
			return -1;
		}

		my $ram;
		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);
		
		if ($os eq 'SunOS' && $osmajor == 4)
		{
			# It is trivial to get this from dmesg, but see my comments
			# at the top of this file as to why we can't trust dmesg.
			# The only other way I could find to get this is from devinfo.
			# Unfortunately it is a little complicated.  Here is a snippit
			# of the devinfo -v output on a machine with 96 megs of ram:
			#
			#   Node 'memory', unit #0 (no driver)
			#     Register specifications:
			#       Bus Type=0x0, Address=0x0, Size=1000000
			#       Bus Type=0x0, Address=0x4000000, Size=1000000
			#       Bus Type=0x0, Address=0x8000000, Size=1000000
			#       Bus Type=0x0, Address=0xc000000, Size=1000000
			#       Bus Type=0x0, Address=0x10000000, Size=1000000
			#       Bus Type=0x0, Address=0x1c000000, Size=1000000
			#
			# If you add up all of the sizes and multiply by 16 you get
			# 96 million.  Divide by a million and there you have it...
			#
			# I'm still not convinced this is reliable though.  In one
			# instance I know this gave the wrong answer and in a couple
			# of others it gave an answer different than dmesg and I'm
			# not sure which is correct.
 
			my $memnode = 0;
			my $memmillions = 0;
			local *DEVINFO;
			open(DEVINFO, "rsh $host /usr/etc/devinfo -v|");
			while(<DEVINFO>)
			{
				if (/Node 'memory'/)
				{
					$memnode = 1;
				}
				elsif ($memnode && /Node/)
				{
					$memnode = 0;
				}
				elsif ($memnode)
				{
					if (/Bus Type/)
					{
						my $junk;
						($junk, $junk, $junk, my $size) = split;
						($junk, my $sizenum) = split(/=/, $size);
						$memmillions += $sizenum * 16;
					}
				}
			}
			close(DEVINFO);
 
			$ram = $memmillions / 1000000;
		}
		elsif ($os eq 'SunOS' && $osmajor == 5)
		{
			local *PRTCONF;
			open(PRTCONF, "rsh $host prtconf|");
			while(<PRTCONF>)
			{
				if (/Memory size/)
				{
					(my $junk, $junk, $ram) = split;
				}
			}
			close(PRTCONF);
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			chop($ram = rsh_noerr($host, '/usr/sam/lbin/getmem'));
		}
		elsif ($os eq 'HP-UX' && $osmajor == 9)
		{
			chop(my $adboutput  = rsh_noerr($host,
				'echo physmem/D | adb /hp-ux /dev/kmem | tail -1'));
			my ($junk, $four_kilobytes) = split(' ', $adboutput);
			$ram = $four_kilobytes / 256;
		}
		else
		{
			$ram = -1;
		}

		unless (defined($ram))
		{
			$ram = -1;
		}

		$cache_ram{$host} = $ram;
		return $ram;
	}
}

#### !!! Needs work !!! ####
sub get_hard_disks
{
	my $host = $_[0];
	my $verbose = 0;

	if (defined($cache_hard_disks{$host}))
	{
		return $cache_hard_disks{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_hard_disks{$host} = -1;
			return -1;
		}

		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);

		# Someday...
		if ($os ne 'SunOS')
		{
			$cache_hard_disks{$host} = -1;
			return -1;
		}

		# A lot of the info about the disks in the machine is gathered from
		# the list of disks that format spits out when you first start it
		# up.  Since we run through that info several times, we go ahead and
		# grab it once so that we don't have to make multiple connections to
		# the machine for the same info.
		#
		# Format is designed to be interactive.  The echo makes it
		# bail out like we want.  Note that format occasionally takes
		# an exceptionally long time to run.  I suspect that this
		# occurs due to hardware problems on the machine.  That is another
		# benefit for only doing this once.

		my @disks = ();
		my %disksize;

		my @formatlines = ();
		local *FORMAT;
		if ($osmajor == 5)
		{
			open(FORMAT, "echo | rsh $host /usr/sbin/format |");
		}
		elsif ($osmajor == 4)
		{                                           
			open(FORMAT, "echo | rsh $host /usr/etc/format |");
		}
		else
		{
			# How'd we get here?
			die "Internal error in \&get_hard_disks";
		}
	
		while(<FORMAT>)
		{
			chop;
			push(@formatlines, $_);
		}
		close(FORMAT);
	
		# First we get all of the available disks
		foreach $_ (@formatlines)
		{
			if (/^\s+\d+\./)
			{
				my ($junk, $disk) = split;
				push(@disks, $disk);
				print "Found disk: $disk\n" if ($verbose);
			}
		}
	
		if ($osmajor == 5)
		{
			my $disk;
			foreach $disk (@disks)
			{
				# Once again stolen from suninstall.  We know which
				# disk we are looking for, but prtvtoc wants a slice as
				# well.  Thus we iterate through slices 0-7 until we
				# find one that works.  It doesn't matter which slice
				# we use, we just need one that exists.
	
				# A possible speed-up would be to just assume slice 2
				# exists and only search if it doesn't...
		
				my ($i, $slice);
				SLICE: for ($i = 0 ; $i < 8 ; $i++)
				{
					# Also, Solaris 2.3 requires that you use the
					# raw device (/dev/rdsk) while 2.4 or greater
					# allow either the raw device or the block
					# device (/dev/dsk).  As such we use the raw
					# device.
					unless (rsh_return($host, "prtvtoc /dev/rdsk/${disk}s$i"))
					{
						$slice = $i;
						last SLICE;
					}
				}
		
				my ($bps, $spc, $cyl);
				open(PRTVTOC, "rsh $host prtvtoc /dev/rdsk/${disk}s${slice}|");
				while(<PRTVTOC>)
				{
					if (/bytes\/sector/)
					{
						(my $junk, $bps) = split;
					}
					elsif (/sectors\/cylinder/)
					{
						(my $junk, $spc) = split;
					}
					elsif (/cylinders/)
					{
						(my $junk, $cyl) = split;
					}
				}
				close(PRTVTOC);
				
				my $mbytes = ($bps * $spc * $cyl) / 1048576;
		
				$disksize{$disk} = $mbytes;
			}
		}
		elsif ($osmajor == 4)
		{
			my $disk;
			foreach $disk (@disks)
			{
				foreach $_ (@formatlines)
				{
					next unless (/^\s+${disk}:/);
	
					# A typical line like what we are looking for:
					# sd0: <SUN1.05 cyl 2036 alt 2 hd 14 sec 72>
					# And I saw one like this once (with the ^[
					# being the actual escape character):
					# sd0: <FUJITSU 2624FA\^[ cyl 1463 alt 2 ...
					# As such, I no longer trust just doing a split
					# and assuming things will be where I expect.
	
					/cyl ([0-9]+) alt [0-9]+ hd ([0-9]+) sec ([0-9]+)/;
					$cyl = $1;
					$heads = $2;
					$sec = $3;
	
					# We assume 512 bytes/sector
					$disksize{$disk} = (512 * $sec * $heads * $cyl) / 1048576;
				}
			}
		}
		else
		{
			# Acck, what happened?
			die "Internal error in \&get_hard_disks"
		}
	
		# For SunOS machines we have to do a little name translation.
		# Namely, we need to map the drive names from SunOS format
		# (sd0, sd1) to Solaris format (c0t3d0, c0t1d0).  But the trick
		# is that we need to know which SCSI ids the various SunOS names
		# map to in order to correctly map them to the Solaris names.
		# The most reliable way to get this is from format.  Here is a
		# snippit of the output from format on a machine with a pretty
		# wacky device name to SCSI id mapping:
		#
		#        0. sd0 at esp0 slave 24
		#           sd0: <FUJITSU 2624FA\cyl 1463 alt 2 hd 11 sec 63>
		#        1. sd1 at esp0 slave 0
		#           sd1: <SUN1.3G cyl 1965 alt 2 hd 17 sec 80>
		#        2. sd2 at esp0 slave 32
		#           sd2: <SUN1.3G cyl 1965 alt 2 hd 17 sec 80>
		#        3. sd4 at esp0 slave 8
		#           sd4: <SUN0424 cyl 1151 alt 2 hd 9 sec 80>
		#        4. sd5 at esp0 slave 16
		#           sd5: <SUN0669 cyl 1614 alt 2 hd 15 sec 54>
		#        5. sd6 at esp0 slave 48
		#           sd6: <SUN4.2G cyl 3880 alt 2 hd 16 sec 135>
		#
		# It turns out that if you divide the slave number (i.e. 24 for
		# sd0) by 8 that is the SCSI id number.  Intuitive, eh?  :)
	
		if ($osmajor == 4)
		{
			my %SUNTOSOL;
	
			foreach $_ (@formatlines)
			{
				next unless (/\s+\d+\./);
				/(sd\d+).*slave\s+(\d+)$/;
				my $disk = $1;
				my $slave = $2;
				my $scsi_id = $slave / 8;
				print "Mapping $disk to c0t${scsi_id}d0\n" if ($verbose);
				$SUNTOSOL{$disk} = "c0t${scsi_id}d0";
			}

			my ($disk, @newdisks);
			foreach $disk (@disks)
			{
				push(@newdisks, $SUNTOSOL{$disk});
			}
			@disks = @newdisks;
	
			foreach $disk (keys(%disksize))
			{
				$disksize{$SUNTOSOL{$disk}} = $disksize{$disk};
			}
		}
	
		my $disk;
		my $return_string = '';
		foreach $disk (sort @disks)
		{
			print "$disk: $disksize{$disk}\n" if ($verbose);
			$return_string .= "$disk=" . int($disksize{$disk}) . "\;";
		}

		# Remove trailing semicolon
		chop($return_string);

		return $return_string;
	}
}

#### !!! Needs work !!! ####
sub get_cpu_manufacturer
{
	my $host = $_[0];

	if (defined($cache_cpu_manufacturer{$host}))
	{
		return $cache_cpu_manufacturer{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_cpu_manufacturer{$host} = -1;
			return -1;
		}

		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);
		my $cpuman;
		my $cpumodel;

		if ($os eq 'SunOS')
		{
			my $command;

			if ($osmajor == 4)
			{
				$command = '/usr/etc/devinfo';
			}
			elsif ($osmajor == 5)
			{
				$command = '/usr/sbin/sysdef -d';
			}
			else
			{
				$command = '';
			}

			# Ok, this is going to sound amazing but here goes.  We're
			# looking for Node entries that are one tab stop in and
			# contain a comma.  Then for Ultras we want the entry that
			# isn't SUNW,ffb and for others we want anything that doesn't
			# start with SUNW.  For example:
			#
			# % sysdef -d | grep "Node '.*,.*'"
			# Node 'SUNW,Ultra-1', unit #-1
			#                 Node 'SUNW,CS4231', unit #-1 (no driver)
			#                 Node 'SUNW,fdtwo', unit #0
			#                 Node 'SUNW,pll', unit #-1 (no driver)
			#                 Node 'SUNW,fas', unit #0
			#                 Node 'SUNW,hme', unit #0
			#                 Node 'SUNW,bpp', unit #-1 (no driver)
			#         Node 'SUNW,UltraSPARC', unit #-1 (no driver)
			#         Node 'SUNW,ffb', unit #0

			if ($command)
			{
				my $model = get_model($host);
				my $output = rsh($host, $command);
				my $cpuinfo;

				MATCH: foreach $match ($output =~ /^\tNode '(\w+\,[\w\-]+)'/gm)
				{
					if ($model =~ /Ultra/)
					{
						# SUNW is ok here but skip the SUNW,ffb
						next MATCH if ($match eq 'SUNW,ffb');
						$cpuinfo = $match;
					}
					else
					{
						next MATCH if ($match =~ /^SUNW/);
						$cpuinfo = $match;
					}
				}

				if ($cpuinfo)
				{
					($cpuman, $cpumodel) = split(',', $cpuinfo);
				}
				else
				{
					$cpuman = -1;
					$cpumodel = -1;
				}
			}
			else
			{
				$cpuman = -1;
				$cpumodel = -1;
			}
		}
		elsif ($os eq 'Linux')
		{
			my $cpuinfo = rsh_noerr($host, 'cat /proc/cpuinfo');

			if ($cpuinfo =~ /^model\s+:\s+(.*)/m)
			{
				$cpumodel = $1;
			}
			else
			{
				$cpumodel = -1;
			}

			if ($cpuinfo =~ /^vendor_id\s+:\s+(.*)/m)
			{
				$cpuman = $1;
			}
			else
			{
				$cpuman = -1;
			}
		}

		$cache_cpu_manufacturer{$host} = $cpuman;
		$cache_cpu_model{$host} = $cpumodel;
		return $cpuman;
	}
}

sub get_cpu_model
{
	my $host = $_[0];

	if (defined($cache_cpu_model{$host}))
	{
		return $cache_cpu_model{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_cpu_model{$host} = -1;
			return -1;
		}

		# get_cpu_manufacturer caches the cpu model for Solaris and
		# Linux.
		get_cpu_manufacturer($host);
		$cache_cpu_model{$host} = -1 unless (defined($cache_cpu_model{$host}));
		return $cache_cpu_model{$host};
	}
}

sub get_cpu_clock_speed
{
	my $host = $_[0];

	if (defined($cache_cpu_clock_speed{$host}))
	{
		return $cache_cpu_clock_speed{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_cpu_clock_speed{$host} = -1;
			return -1;
		}

		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);
		my $cpuspeed;

		if ($os eq 'SunOS' && $osmajor == 4)
		{
			# Hopefully the CPU section always comes last...
			my $output = rsh_noerr($host,
					'/usr/etc/devinfo -pv | grep clock-frequency | tail -1');
			my ($junk, $hexspeed) = split(' ', $output);
			$cpuspeed = hex($hexspeed) / 1000000;
		}
		elsif ($os eq 'SunOS' && $osmajor == 5)
		{
			my $output = rsh_noerr($host, '/usr/sbin/psrinfo -v');
			if ($output =~ /operates at (\d+) MHz/)
			{
				$cpuspeed = $1;
			}
			else
			{
				$cpuspeed = -1;
			}
		}
		elsif ($os eq 'HP-UX' && $osmajor == 9)
		{
			# Pretty much straight from the HP-UX FAQ
			chop(my $adboutput = rsh_noerr($host,
				'echo itick_per_tick/D | adb /hp-ux /dev/kmem | tail -n1'));
			my ($junk, $hectohertz) = split(' ', $adboutput);
			$cpuspeed = $hectohertz / 10000;
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			# Slightly modified version of the above command to reflect
			# the different location of the kernel in HP-UX 10.x
			chop(my $adboutput = rsh_noerr($host,
				'echo itick_per_tick/D | adb /stand/vmunix /dev/kmem | ' .
					'tail -n1'));
			my ($junk, $hectohertz) = split(' ', $adboutput);
			$cpuspeed = $hectohertz / 10000;
		}
		else
		{
			$cpuspeed = -1;
		}

		$cache_cpu_clock_speed{$host} = $cpuspeed;
		return $cpuspeed;
	}
}

# Return the model (i.e. SPARCstation-5 or C110)
sub get_model
{
	my $host = $_[0];

	if (defined($cache_model{$host}))
	{
		return $cache_model{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_model{$host} = -1;
			return -1;
		}

		my $os = get_os($host);
		my $osnver = get_os_numeric_version($host);
		my $osmajor = get_os_major_version($host);
		my $model;

		if ($os eq 'SunOS' && $osmajor == 4)
		{
			my $output = rsh_noerr($host, '/usr/etc/devinfo -v | head -1');

			# $output looks like:
			#
			# Node 'SUNW,SPARCstation-5', unit #0 (no driver)
			#
			# We want the SUNW,SPARCstation-5 part.

			if ($output =~ /'(.*)'/)
			{
				$model = $1;
			}
			else
			{
				$model = -1;
			}
		}
		elsif ($os eq 'SunOS' && $osnver >= 5 && $osnver < 5.5)
		{
			# I'm going to bet that this is always the 5th line
			# from prtconf and hope it works...
			chop($model = rsh_noerr($host,
									'/usr/sbin/prtconf | head -5 | tail -1'));
		}
		elsif ($os eq 'SunOS' && $osnver >= 5.5)
		{
			chop($model = rsh_noerr($host, '/usr/bin/uname -i'));
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			chop($model = rsh($host, 'model'));
			#chop(my $modeloutput = rsh($host, 'model'));
			#(my $junk, my $junk, $model) = split(/\//, $modeloutput);
		}
		else
		{
			$model = -1;
		}
		
		# Strip the SUNW, out of the model name
		$model =~ s/^SUNW,// if ($os eq 'SunOS');

		unless (defined($model))
		{
			$model = -1;
		}

		$cache_model{$host} = $model;
		return $model;
	}
}

sub get_has_dnj_file
{
	my $host = $_[0];

	if (defined($cache_has_dnj_file{$host}))
	{
		return $cache_has_dnj_file{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_has_dnj_file{$host} = -1;
			return -1;
		}

		if (rsh_file_exists($host, '/usr/local/do_not_jumpstart'))
		{
			$cache_has_dnj_file{$host} = $TRUE;
			return $TRUE;
		}
		else
		{
			$cache_has_dnj_file{$host} = $FALSE;
			return $FALSE;
		}
	}
}

# Returns the name of the standard image this machine was created from
sub get_image
{
	my $host = $_[0];

	if (defined($cache_image{$host}))
	{
		return $cache_image{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_image{$host} = -1;
			return -1;
		}

		my $image;
		my $version;

		# The old format for these files is /usr/local/<image>_version_<ver>
		# I.e. for a 2.1 client it would be /usr/local/client_version_2.1

		my $lsoutput = rsh_noerr($host,
							'ls /usr/local/*_version_* | grep -v genlp');
		if ($lsoutput)
		{
			my (@fileparts) = split(m#/#, $lsoutput);
			my $file = $fileparts[$#fileparts];
			if ($file)
			{
				chop($file);
				($image, my $junk, $version) = split(/_/, $file);
			}
		}
		else
		{
			# Try the new format,
			# /usr/local/Profile_<image>_<version>_<subimage>
			# I.e. for a 2.2 standard client jumpstarted with the regular
			# js subimage it would be /usr/local/Profile_stdclient_2.2_js

			my $lsoutput = rsh_noerr($host, 'ls /usr/local/Profile_*');

			if ($lsoutput)
			{
				my (@fileparts) = split(m#/#, $lsoutput);
				my $file = $fileparts[$#fileparts];
				if ($file)
				{
					chop($file);
					(my $junk, $image, $version, my $subimage) =
						split(/_/, $file);
				}
			}
			else
			{
				$image = '';
				$version = '';
			}
		}

		$image = -1 unless (defined($image));
		$version = -1 unless (defined($version));

		$cache_image{$host} = $image;
		# get_image_version depends on this happening so remember to
		# change it if you change this.
		$cache_image_version{$host} = $version;
		return $image;
	}
}

sub get_image_version
{
	my $host = $_[0];

	if (defined($cache_image_version{$host}))
	{
		return $cache_image_version{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_image_version{$host} = -1;
			return -1;
		}

		# get_image caches the image version as a handy side effect
		get_image($host);
		$cache_image_version{$host} = -1
			unless (defined($cache_image_version{$host}));
		return $cache_image_version{$host};
	}
}

sub get_sendmail_version
{
	my $host = $_[0];

	if (defined($cache_sendmail_version{$host}))
	{
		return $cache_sendmail_version{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_sendmail_version{$host} = -1;
			return -1;
		}

		# Straight out of the bat book
		my $sendmailoutput = rsh($host,
								'/usr/lib/sendmail -d0.1 -bt < /dev/null');
		my $sendmailversion;
		if ($sendmailoutput =~ /Version ([-\w.]+)/)
		{
			$sendmailversion = $1;
		}
		else
		{
			$sendmailversion = -1;
		}

		$cache_sendmail_version{$host} = $sendmailversion;
		return $sendmailversion;
	}
}

sub get_sendmail_mailhub
{
	my $host = $_[0];

	if (defined($cache_sendmail_mailhub{$host}))
	{
		return $cache_sendmail_mailhub{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_sendmail_mailhub{$host} = -1;
			return -1;
		}

		my $os = get_os($host);
		my $osmajor = get_os_major_version($host);
		my $sendmailcf;
		my $mailhub;
		my $smartrelay;

		if ($os eq 'SunOS' && $osmajor == 4)
		{
			$sendmailcf = '/etc/sendmail.cf';
		}
		elsif ($os eq 'SunOS' && $osmajor == 5)
		{
			$sendmailcf = '/etc/mail/sendmail.cf';
		}
		elsif ($os eq 'HP-UX' && $osmajor == 9)
		{
			$sendmailcf = '/usr/lib/sendmail.cf';
		}
		elsif ($os eq 'HP-UX' && $osmajor == 10)
		{
			$sendmailcf = '/etc/mail/sendmail.cf';
		}
		else
		{
			$sendmailcf = '';
		}

		if ($sendmailcf)
		{
			# Look for either the mailhub or smart host
			my $grepoutput =
					rsh_nosplit($host, "egrep '^(DH|DS)' $sendmailcf");

			if ($grepoutput)
			{
				if ($grepoutput =~ /DH(\w+)/)
				{
					$mailhub = $1;
				}
				else
				{
					$mailhub = '';
				}

				if ($grepoutput =~ /DS(\w+)/)
				{
					$smarthost = $1;
				}
				else
				{
					$smarthost = '';
				}
			}
		}

		$mailhub = -1 unless (defined($mailhub));
		$smarthost = -1 unless (defined($smarthost));

		$cache_sendmail_mailhub{$host} = $mailhub;
		$cache_sendmail_smarthost{$host} = $smarthost;
		return $mailhub;
	}
}

sub get_sendmail_smarthost
{
	my $host = $_[0];

	if (defined($cache_sendmail_smarthost{$host}))
	{
		return $cache_sendmail_smarthost{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_sendmail_smarthost{$host} = -1;
			return -1;
		}

		get_sendmail_mailhub($host);
		$cache_sendmail_smarthost{$host} = -1
			unless (defined($cache_sendmail_smarthost{$host}));
		return $cache_sendmail_smarthost{$host};
	}
}

# Should return something that is specific to that particular machine.
# Returns output from hostid on SunOS, output from uname -i on HP-UX,
# hostid on Linux
# Note that hostid on SunOS and Linux gives its output in hex, uname -i
# on HP-UX gives its output in decimal.  I don't try to convert them.
sub get_hostid
{
	my $host = $_[0];

	if (defined($cache_hostid{$host}))
	{
		return $cache_hostid{$host};
	}
	else
	{
		unless (get_can_ping($host) && get_can_rsh($host))
		{
			$cache_hostid{$host} = -1;
			return -1;
		}

		my $os = get_os($host);

		my $hostid;
		if ($os eq 'SunOS' || $os eq 'Linux')
		{
			# The hostid command is usually found in /usr/bin.  However,
			# I did have one machine that only had it in /usr/ucb.
			if (rsh_file_exists($host, '/usr/bin/hostid'))
			{
				chop($hostid = rsh($host, '/usr/bin/hostid'));
			}
			elsif (rsh_file_exists($host, '/usr/ucb/hostid'))
			{
				chop($hostid = rsh($host, '/usr/ucb/hostid'));
			}
			else
			{
				$hostid = -1;
			}
		}
		elsif ($os eq 'HP-UX')
		{
			chop($hostid = rsh($host, 'uname -i'));
		}
		else
		{
			$hostid = -1;
		}

		$cache_hostid{$host} = $hostid;
		return $hostid;
	}
}

return 1;

